import unittest
from Products.CMFCore.utils import getToolByName
from collective.langMailHost.tests.base import MailHostTestCase

class TestSetup(MailHostTestCase):

    def afterSetUp(self):
        self.properties = getToolByName(self.portal, 'portal_properties')

    ## Propertiestool.xml
    def test_lang_charset(self):
        self.failUnless('ja|iso-2022-jp', self.properties.mailhost_properties.getProperty('lang_charset'))

    def test_is_member_lang_effective(self):
        self.assertEquals(True, self.properties.mailhost_properties.getProperty('is_member_lang_effective'))

#    def test_is_site_lang_effective(self):
#        self.assertEquals(True, self.properties.mailhost_properties.getProperty('is_site_lang_effective'))

def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestSetup))
    return suite



from Acquisition import aq_inner
from zope.interface import implements
from Products.CMFCore.utils import getToolByName
from collective.langMailHost.interfaces import (
    ILangCharset,
)

class LangCharset(object):
    implements(ILangCharset)
    def __init__(self, context):
        self.context = context

    def lang_charset(self):
        context = aq_inner(self.context)
        properties = getToolByName(context, 'portal_properties')
        if len(properties.mailhost_properties.getProperty('lang_charset')) != 0:
            lang_charset_tuples = [
                lc.split('|') for lc in properties.mailhost_properties.getProperty('lang_charset')
            ]
            lang_charset = dict(lang_charset_tuples)
            return lang_charset

    def effective_lang(self):
        context = aq_inner(self.context)
        membership = getToolByName(context, 'portal_membership')
        properties = getToolByName(context, 'portal_properties')
        is_member_lang = properties.mailhost_properties.getProperty('is_member_lang_effective')
        member_lang = membership.getAuthenticatedMember().getProperty('language')
        if is_member_lang and member_lang != '' and member_lang is not None:
            return member_lang
        else:
            language_tool = getToolByName(context, 'portal_languages')
            language_bindings = language_tool.getLanguageBindings()
            return language_bindings[0]

    def effective_charset(self):
        lang = self.effective_lang()
        if lang:
            return self.lang_charset().get(lang)



import re
from base64 import b64encode
from AccessControl import Unauthorized#, allow_module
from smtplib import SMTPRecipientsRefused
from zope.component import getUtility
from Acquisition import aq_inner
from Products.CMFDefault.utils import checkEmailAddress
from Products.CMFDefault.exceptions import EmailAddressInvalid
from Products.CMFCore.interfaces import ISiteRoot
from Products.CMFCore.utils import getToolByName
from collective.langMailHost.interfaces import ILangCharset
from AccessControl import ClassSecurityInfo
from Products.CMFPlone.RegistrationTool import RegistrationTool
from zope.i18nmessageid import MessageFactory
_ = MessageFactory('shopinmall')

class RegistrationTool(RegistrationTool):

    security = ClassSecurityInfo()

    security.declarePublic('mailPassword')
    def mailPassword(self, forgotten_userid, REQUEST):
        """ Wrapper around mailPassword """
        membership = getToolByName(self, 'portal_membership')
        if not membership.checkPermission('Mail forgotten password', self):
            raise Unauthorized, "Mailing forgotten passwords has been disabled"

        utils = getToolByName(self, 'plone_utils')
        member = membership.getMemberById(forgotten_userid)

        if member is None:
            raise ValueError, 'The username you entered could not be found'

        # assert that we can actually get an email address, otherwise
        # the template will be made with a blank To:, this is bad
        email = member.getProperty('email')
        if not email:
            raise ValueError('That user does not have an email address.')
        else:
            # add the single email address
            if not utils.validateSingleEmailAddress(email):
                raise ValueError, 'The email address did not validate'
        check, msg = _checkEmail(email)
        if not check:
            raise ValueError, msg

        # Rather than have the template try to use the mailhost, we will
        # render the message ourselves and send it from here (where we
        # don't need to worry about 'UseMailHost' permissions).
        reset_tool = getToolByName(self, 'portal_password_reset')
        reset = reset_tool.requestReset(forgotten_userid)
        urltool = getToolByName(self, 'portal_url')
        portal = urltool.getPortalObject()
        email_charset = ILangCharset(portal).effective_charset() or portal.getProperty('email_charset', 'UTF-8')

        subject = portal.translate(msgid=u'mailtemplate_subject_resetpasswordrequest', domain='passwordresettool')
        subject = b64encode(subject.encode(email_charset))
        subject = '=?%s?b?%s?=' %(email_charset, subject)
        mail_text = self.mail_password_template( self
                                               , REQUEST
                                               , member=member
                                               , reset=reset
                                               , password=member.getPassword()
                                               , charset=email_charset
                                               )
        if isinstance(mail_text, unicode):
            mail_text = mail_text.encode(email_charset, 'replace')
        host = self.MailHost
        try:
            host.send(mail_text, subject=subject)

            return self.mail_password_response( self, REQUEST )
        except SMTPRecipientsRefused:
            # Don't disclose email address on failure
            raise SMTPRecipientsRefused('Recipient address rejected by server')


    security.declarePublic('registeredNotify')
    def registeredNotify(self, new_member_id):
#    def registeredNotify(self):
        """ Wrapper around registeredNotify """
#        REQUEST = self.request
#        new_member_id = REQUEST['username']
#        context = aq_inner(self.context)
        membership = getToolByName( self, 'portal_membership' )
        utils = getToolByName(self, 'plone_utils')
        member = membership.getMemberById( new_member_id )

        if member and member.getProperty('email'):
            # add the single email address
            if not utils.validateSingleEmailAddress(member.getProperty('email')):
                raise ValueError, 'The email address did not validate'

        email = member.getProperty( 'email' )
        try:
            checkEmailAddress(email)
        except EmailAddressInvalid:
            raise ValueError, 'The email address did not validate'

        pwrt = getToolByName(self, 'portal_password_reset')
        reset = pwrt.requestReset(new_member_id)

        urltool = getToolByName(self, 'portal_url')
        portal = urltool.getPortalObject()
        encoding = ILangCharset(portal).effective_charset() or getUtility(ISiteRoot).getProperty('email_charset')

#        subject = _(
#            u'mailtemplate_user_account_info',
#            u"User Account Information for ${portal_name}",
#            mapping={'portal_name' : portal.Title()}
#        )
        subject = portal.translate(msgid=u'mailtemplate_user_account_info', domain='passwordresettool', mapping={'portal_name' : portal.Title()})
        subject = b64encode(subject.encode(encoding))
        subject = '=?%s?b?%s?=' %(encoding, subject)


#Subject: <span i18n:domain="passwordresettool" i18n:translate="mailtemplate_user_account_info" tal:omit-tag="">User Account Information for <span i18n:name="portal_name" tal:omit-tag="" tal:content="python:here.portal_url.getPortalObject().Title()" /></span>

        # Rather than have the template try to use the mailhost, we will
        # render the message ourselves and send it from here (where we
        # don't need to worry about 'UseMailHost' permissions).

        mail_text = self.registered_notify_template( self
                                                   , self.REQUEST
                                                   , member=member
                                                   , reset=reset
                                                   , email=email
                                                   , charset=encoding
                                                   )

        if isinstance(mail_text, unicode):
            mail_text = mail_text.encode(encoding, 'replace')

        host = self.MailHost

        host.send(mail_text, subject=subject)

        return self.mail_password_response( self, self.REQUEST )


_TESTS = ( ( re.compile("^[0-9a-zA-Z\.\-\_\+\']+\@[0-9a-zA-Z\.\-]+$")
           , True
           , "Failed a"
           )
         , ( re.compile("^[^0-9a-zA-Z]|[^0-9a-zA-Z]$")
           , False
           , "Failed b"
           )
         , ( re.compile("([0-9a-zA-Z_]{1})\@.")
           , True
           , "Failed c"
           )
         , ( re.compile(".\@([0-9a-zA-Z]{1})")
           , True
           , "Failed d"
           )
         , ( re.compile(".\.\-.|.\-\..|.\.\..|.\-\-.")
           , False
           , "Failed e"
           )
         , ( re.compile(".\.\_.|.\-\_.|.\_\..|.\_\-.|.\_\_.")
           , False
           , "Failed f"
           )
         , ( re.compile(".\.([a-zA-Z]{2,3})$|.\.([a-zA-Z]{2,4})$")
           , True
           , "Failed g"
           )
         )

def _checkEmail( address ):
    for pattern, expected, message in _TESTS:
        matched = pattern.search( address ) is not None
        if matched != expected:
            return False, message
    return True, ''




from Acquisition import aq_inner
from zope.component.interfaces import ComponentLookupError
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.utils import safe_unicode
from plone.app.contentrules.actions.mail import MailActionExecutor
from collective.langMailHost.interfaces import ILangCharset

class LangMailActionExecutor(MailActionExecutor):
    """The executor for this action.
    """
#    implements(IExecutable)
#    adapts(Interface, IMailAction, Interface)

#    def __init__(self, context, element, event):
#        self.context = context
#        self.element = element
#        self.event = event

    def __call__(self):
        recipients = [str(mail.strip()) for mail in \
                      self.element.recipients.split(',')]
        mailhost = getToolByName(aq_inner(self.context), "MailHost")
        if not mailhost:
            raise ComponentLookupError, 'You must have a Mailhost utility to \
execute this action'

        source = self.element.source
        urltool = getToolByName(aq_inner(self.context), "portal_url")
        portal = urltool.getPortalObject()
        email_charset = ILangCharset(portal).effective_charset() or portal.getProperty('email_charset', 'UTF-8')
        if not source:
            # no source provided, looking for the site wide from email
            # address
            from_address = portal.getProperty('email_from_address')
            if not from_address:
                raise ValueError, 'You must provide a source address for this \
action or enter an email in the portal properties'
            from_name = portal.getProperty('email_from_name')
            source = "%s <%s>" % (from_name, from_address)

        obj = self.event.object
        event_title = safe_unicode(obj.Title())
        event_url = obj.absolute_url()
        message = self.element.message.replace("${url}", event_url)
        message = message.replace("${title}", event_title)

        subject = self.element.subject.replace("${url}", event_url)
        subject = subject.replace("${title}", event_title)

        for email_recipient in recipients:
            mailhost.secureSend(message, email_recipient, source,
                                subject=subject, subtype='plain',
                                charset=email_charset, debug=False,
                                From=source)
        return True




"""
Mail related doctests
"""

import unittest
from zope.testing import doctest

from Testing.ZopeTestCase import FunctionalDocFileSuite
#from Products.CMFPlone.tests import PloneTestCase

from Products.CMFPlone.tests.utils import MockMailHost

OPTIONFLAGS = (doctest.ELLIPSIS |
               doctest.NORMALIZE_WHITESPACE |
               doctest.REPORT_ONLY_FIRST_FAILURE)
from collective.langMailHost.tests.base import MailHostFunctionalTestCase

#class MockMailHostTestCase(PloneTestCase.FunctionalTestCase):
class MockMailHostTestCase(MailHostFunctionalTestCase):

    def afterSetUp(self):
        self.portal._original_MailHost = self.portal.MailHost
        self.portal.MailHost = MockMailHost('MailHost')

    def beforeTearDown(self):
        self.portal.MailHost = self.portal._original_MailHost


def test_suite():
    return unittest.TestSuite((
        FunctionalDocFileSuite('tests/mails.txt',
                               optionflags=OPTIONFLAGS,
                               package='collective.langMailHost',
                               test_class=MockMailHostTestCase),
        ))



from zope.interface import Interface
from AccessControl import allow_class

class ILangCharset(Interface):
    def lang_charset():
        """Returns lang:charset dictionary from mailhost_properties."""

    def effective_lang():
        """Returns language of member's choice."""

    def effective_charset():
        """Returns charset from the language."""

allow_class(ILangCharset)

#class ISendTo(Interface):
#    def __call__():
#        """Send Mail from sendto script."""



from setuptools import setup, find_packages
import os

def read(*rnames):
    return open(os.path.join(os.path.dirname(__file__), *rnames)).read()

version = read('collective', 'langMailHost', 'version.txt')[:-1]

long_description = (
    open("README.txt").read() + "\n" +
    open(os.path.join("docs", "HISTORY.txt")).read() + "\n" +
    open(os.path.join("docs", "INSTALL.txt")).read()
)

setup(name='collective.langMailHost',
      version=version,
      description="This package aims to provide language dependent character set for sending emails form Plone site.",
      long_description=long_description,
      # Get more strings from http://www.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        "Framework :: Plone",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        ],
      keywords='',
      author='Taito Horiuchi',
      author_email='taito.horiuchi@gmail.com',
      url='http://pypi.python.org/pypi/collective.langMailHost',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['collective'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          # -*- Extra requirements: -*-
      ],
      entry_points="""
      # -*- Entry points: -*-

      [distutils.setup_keywords]
      paster_plugins = setuptools.dist:assert_string_list

      [egg_info.writers]
      paster_plugins.txt = setuptools.command.egg_info:write_arg
      """,
      paster_plugins = ["ZopeSkel"],
      )



import unittest
import doctest
from Testing import ZopeTestCase as ztc
from collective.langMailHost.tests import base
from Products.CMFCore.utils import getToolByName

class TestSetup(base.MailHostFunctionalTestCase):

    def afterSetUp( self ):
        """After SetUp"""
        portal = self.portal
        language_tool = getToolByName(portal, 'portal_languages')
        language_tool.addSupportedLanguage('ja')

def test_suite():
    return unittest.TestSuite([

        # Integration tests for adapters.
        ztc.FunctionalDocFileSuite(
            'tests/functional/adapter.txt', package='collective.langMailHost',
            test_class=TestSetup,
            optionflags=doctest.REPORT_ONLY_FIRST_FAILURE | doctest.NORMALIZE_WHITESPACE | doctest.ELLIPSIS),

            ])

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')



from Products.Five import zcml
from Products.Five import fiveconfigure
from Testing import ZopeTestCase as ztc
from Products.PloneTestCase import PloneTestCase as ptc
from Products.PloneTestCase.layer import onsetup

@onsetup
def setup_langMailHost():

    fiveconfigure.debug_mode = True
    import collective.langMailHost
    zcml.load_config('configure.zcml', collective.langMailHost)
    fiveconfigure.debug_mode = False

    ztc.installPackage('collective.langMailHost')

# The order here is important: We first call the (deferred) function which
# installs the products we need for the Mall package. Then, we let 
# PloneTestCase set up this product on installation.

setup_langMailHost()
ptc.setupPloneSite(products=['collective.langMailHost'])

class MailHostTestCase(ptc.PloneTestCase):
    """Base class used for test cases
    """

#    def afterSetUp( self ):
#        """Code that is needed is the afterSetUp of both test cases.
#        """

class MailHostFunctionalTestCase(ptc.FunctionalTestCase):
    """Test case class used for functional (doc-)tests
    """
